; ********************************************************************************
; Generic RGBY Standalone 4-Channel LED driver
; Release Code 1.3.0.3 (19/09/2013)
; Pete Griffiths (c) 2013
; info@picprojects.org.uk
; Written and assembled using MPLAB v8.86
;
; -----------------------------------------------------------------------------------
; *** IMPORTANT ***
;
cDriveLevel     equ              1              ; 0 = output low turns LED on
                                                ; 1 = output high turns LED on
           
; -----------------------------------------------------------------------------------
; Code to fade between colours is completely rewritten to use variable rate of change
; so that all three LEDs arrive at their new levels simultaneously.
;
; This code will run on 12F629 / 12F675 and 12F683. Select the desired processor
; from the Configure - Select Device menu option in MPLAB IDE, then use the
; Project - Quickbuild option to create the .HEX output file.
; 
; Look for errors during assembly. In particular errors in the SequenceData.inc
; file that indicate either a missing comma in the data, or more data than the 
; chosen PIC has available program memory.
; 
; Be carefull to enter the data in the SequenceData.inc file correctly.  The program
; is only smart enough to work out where your sequences start and finish if you enter
; it in the correct format.  Errors in the data may cause the code to crash.
;
; ********************************************************************************
; Note: For 12F629 and 12F675
; All timings are based on the internal 4Mhz PIC oscillator.  The OSCCAL value is read
; from program memory at 0x3FF and written to the OSCAL register.  This value must be
; present and correct. 
;
;             -=# IF IT IS MISSING THE CODE WILL NOT RUN CORRECTLY #=-
; See http://picprojects.org.uk/projects/recal/ if you need to recalibrate the PIC
;
; This does not apply to 12F683 as it doesn't use this method to calibrate the oscillator
;
; ********************************************************************************
; Pinout connections 
;                            ___ ___
;              led2 output  -|1 ^ 18|- led1 output
;              led3 output  -|2   17|- in push button
;                           -|3   16|- led5 output
;                           -|4   15|- led4 output
;              Vss          -|5   14|- Vdd    
;              led6 output  -|6   13|- led13 output 
;              led7 output  -|7   12|- led12 output
;              led8 output  -|8   11|- led11 output 
;              led9 output  -|9   10|- led10 output
 ;                            
;  Ensure that MCLR is pulled to Vdd using 1K resistor for 12F629/675. 
;
; Sw1 performs multiple functions.
;  1. single press to hold / run displayed colour sequence
;  2. double press to step to next sequence
;  3. press and hold >1.5 seconds to enter sleep mode
;     in sleep mode press, hold for >1.5 seconds then release to wake from sleep
;
;  note :double press is like double clicking a mouse button when using a PC.
;
;
; ********************************************************************************
; Revision information
; genrgbsa_main.asm version 3.0.0
;  First release
;
; genrgbsa_main.asm version 3.0.1
;  Minor corrections to the comments in this file
;
; genrgbsa_main.asm version 3.0.2
;  Fixed problem with state not being saved when entering sleep mode
;  Fixed problem saving state when using 12F683
;
; genrgbsa_main.asm version 3.0.3
;  Modified PWM function to reduce flicker at low brightness levels when fading
;  
; genrgbsa_main_4ch.asm version 1.3.0.3
;  Modified entire code base to add support for a fourth colour channel. RGBY
;  
;  
;
;
; ********************************************************************************

  IFDEF  __16F628
                #include "p16f628.inc"
                #define ADCpresent
  ENDIF

;          
;  IFDEF  __12F675
;                #include "p12f675.inc"
;                #define ADCpresent
;  ENDIF
;
;  IFDEF  __12F629
;                #include "p12f629.inc"
;                
;  ENDIF

  IFDEF __16F628                
     __CONFIG  0x3D18  
;  ELSE
;     __CONFIG       _CP_OFF & _WDT_OFF & _BODEN_OFF & _PWRTE_ON & _INTRC_OSC_NOCLKOUT & _MCLRE_ON & _CPD_OFF
  ENDIF
  
                errorlevel -302 ; suppress banksel warning messages
       
; Program data memory variables
; --------------------------------------------
;          
                cblock          0x20 
                                 
                 led1dir           ; led1 fade direction
	   			 led2dir           ; led2 fade direction
                 led3dir           ; led3 fade direction
                 led4dir           ; led4 fade direction
				 led5dir           ; led5 fade direction
				 led6dir           ; led6 fade direction
				 led7dir           ; led7 fade direction
				 led8dir           ; led8 fade direction
				 led9dir           ; led9 fade direction
				 led10dir          ; led10 fade direction
				 led11dir          ; led11 fade direction
				 led12ir           ; led12 fade direction
				 led13dir          ; led13 fade direction
                 led1dif           ; absolute difference between led1 now and next
                 led2dif           ; absolute difference between led2 now and next
                 led3dif           ; absolute difference between led3 now and next
                 led4dif           ; absolute difference between led4 now and next
                 led5dif           ; absolute difference between led5 now and next
                 led6dif           ; absolute difference between led6 now and next
                 led7dif           ; absolute difference between led7 now and next
                 led8dif           ; absolute difference between led8 now and next
                 led9dif           ; absolute difference between led9 now and next
                 led10dif          ; absolute difference between led10 now and next
                 led11dif          ; absolute difference between led11 now and next
                 led12dif          ; absolute difference between led12 now and next
                 led13dif          ; absolute difference between led13 now and next
                 led1cnt           ; fade step counter led1
                 led2cnt           ; fade step counter led2
                 led3cnt           ; fade step counter led3
                 led4cnt           ; fade step counter led4
                 led5cnt           ; fade step counter led5
                 led6cnt           ; fade step counter led6
                 led7cnt           ; fade step counter led7
                 led8cnt           ; fade step counter led8
                 led9cnt           ; fade step counter led9
                 led10cnt          ; fade step counter led10
                 led11cnt          ; fade step counter led11
                 led12cnt          ; fade step counter led12
                 led13cnt          ; fade step counter led13
                 CntBase           ; fade base count holding variable
                 led1pwmWkg        ; temp working variable for PWM
                 led2pwmWkg        ; temp working variable for PWM
                 led3pwmWkg        ; temp working variable for PWM
                 led4pwmWkg        ; temp working variable for PWM
                 led5pwmWkg        ; temp working variable for PWM
                 led6pwmWkg        ; temp working variable for PWM
                 led7pwmWkg        ; temp working variable for PWM
                 led8pwmWkg        ; temp working variable for PWM
                 led9pwmWkg        ; temp working variable for PWM
                 led10pwmWkg       ; temp working variable for PWM
                 led11pwmWkg       ; temp working variable for PWM
                 led12pwmWkg       ; temp working variable for PWM
                 led13pwmWkg       ; temp working variable for PWM
                 fadeflag          ; fade complete flag variable
                 pwmramp           ; pwm ramp counter
                 driveLevel        ; select drive level
                 sbl               ; Sequence Base Low
                 sbh               ; Sequence Base High
                 seql              ; Current Sequence data pointer Low
                 seqh              ; Current Sequence data pointer High
                 currentSeqNo      ; Current Sequence No
                 highSeqNo         ; Highest Sequence No
                 rate              ; Fade rate
                 hold              ; Sequence hold time
                 led1new           ; led1 data
                 led2new           ; led2 data
                 led3new           ; led3 data
                 led4new           ; led4 data
                 led5new           ; led5 data
                 led6new           ; led6 data
                 led7new           ; led7 data
                 led8new           ; led8 data
                 led9new           ; led9 data
                 led10new          ; led10 data
                 led11new          ; led11 data
                 led12new          ; led12 data
                 led13new          ; led13 data
         
                 seqSwCnt       ; sequence switch counter
                 seqSwDblClk    ; seqence switch double click counter
                 saveCntPS      ; save to EERPOM counter prescaler
                 fadeTimer      ; fade rate timer counter
                 holdTimerPS    ; hold timer prescaler
                 holdTimerHi    ; hold timer counter

	   base:6         ; eeprom save data buffer
      	   saveCnt        ; save to EEPROM counter 
                 savedSeqNo     ; saved Sequence No read from EEPROM
                 freeze         ; SW1 run/freeze flag
                 ess            ; eeprom save state flag                                 
                 FSRsave        ; used to save FSR reg which is usedby two functions.
                 led1pwm        ; led1 pwm level
                 led2pwm        ; led2 pwm level
                 led3pwm        ; led3 pwm level
                 led4pwm        ; led4 pwm level
                 led5pwm        ; led5 pwm level
                 led6pwm        ; led6 pwm level
                 led7pwm        ; led7 pwm level
                 led8pwm        ; led8 pwm level                 led1pwm        ; led1 pwm level
                 led9pwm        ; led9 pwm level
                 led10pwm       ; led10 pwm level
                 led11pwm       ; led11 pwm level
                 led12pwm       ; led12 pwm level
                 led13pwm       ; led13 pwm level

	endc



                
;----------------------------------------------------------
; program Constants - DO NOT change them
cTIMER          equ             .179          ; Timer 0 constant 179 original
                                                ; for 5mS timer tick

cHOLDPRESCALER  equ             .14             ; rgb hold counter prescaler
                                                ; 5mS x Prescaler x holdtime
                                                                
cSAVETIMER      equ             .8              ; time after last seq switch change
                                                ; before saving to EEPROM
                                                ; in increments of 1.28S

;----------------------------------------------------------
; GPIO port variables - DO NOT change them
seqSw           equ             0               ;in
led1            equ             1               ;out
led2            equ             2               ;out
led3            equ             3               ;out
led4            equ             4               ;out
led5            equ             5               ;out
led6            equ             6  				;out
led7            equ             7  				;out
led8            equ             8  				;out
led9            equ             9  				;out
led10           equ             10 				;out
led11           equ             11 				;out
led12           equ             12 				;out
led13           equ             13 				;out
                
;----------------------------------------------------------
;
;
; Bank Select macro instructions
#define         setbank0        bcf             STATUS,RP0      ; Sel Bank 0
#define         setbank1        bsf             STATUS,RP0      ; Sel Bank 1
                
                org 0x000
                
                include         genrgbsa_start_4ch.inc
                include         genrgbsa_func_4ch.inc
                include         genrgbsa_tout_4ch.inc
                radix decimal
_base
                include         SequenceData_4ch.inc



;
	org 0x2110
;               ------------------------------------------------------------------------
 ;               de "genrgbsa_main_4ch.asm 19/09/2013 R1.3.0.3"  
;                de "(c) Pete Griffiths 2006-2013 "
 ;               de "info@picprojects.org.uk"
;               ------------------------------------------------------------------------
                
                ;fill 00, 0x3FF-$
                
                end

